﻿namespace pipeline
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Data;
    using System.Drawing;
    using System.Linq;
    using System.Xml.Linq;
    using System.Text;
    using System.Windows.Forms;
    using System.Net;
    using System.Net.Sockets;
    using System.Collections.Specialized;
    using System.IO;
    using System.Diagnostics;
    using System.Threading;

    public partial class Admin : Form
    {
        private TcpClient _tcp = new TcpClient();

        public Admin()
        {
            InitializeComponent();
            _Disconnect();
        }

        private void _Connect()
        {
            // Hide the button and drop down list.
            connect.Visible = false;
            servers.Visible = false;

            // Show the Dashboard control.
            var dashboard = new Dashboard(_tcp.GetStream());
            dashboard.VisibleChanged += dashboard_VisibleChanged;
            dashboard.Dock = DockStyle.Fill;
            content.Controls.Add(dashboard);
        }

        void dashboard_VisibleChanged(object sender, EventArgs e)
        {
            // Close the Dashoard component.
            content.Controls.Remove(sender as Control);
            _Disconnect();
        }

        private void _Disconnect()
        {
            // Restore the offline mode.
            _tcp.Close();

            connect.Visible = true;
            connect.Enabled = true;
            servers.Visible = true;

            statusLabel.Text = "Disconnected.";
        }


        private void Gui_Load(object sender, EventArgs e)
        {
            // Список адресов серверов можно указать в файле .config
            if (Properties.Settings.Default.servers.Count == 0)
            {
                MessageBox.Show("Failed to read config!");
                Close();
                return;
            }

            // Заполняем выпадающий список серверов.
            foreach (var ip in Properties.Settings.Default.servers)
            {
                servers.Items.Add(ip.Clone());
            }

            // Выбираем первый адрес сервера по умолчанию.
            servers.SelectedIndex = 0;
        }

        
        private void connect_Click(object sender, EventArgs e)
        {
            var me = sender as Control;
            me.Enabled = false;
            
            // Запоминаем выбранный сервер.
            me.Tag = servers.SelectedIndex;

            try
            {
                // Сбрасываем цвет текста у статусной строки.
                var autoColor = Color.Empty;
                if (this.Tag != null)
                {
                    autoColor = (Color)this.Tag;
                }
                else
                {
                    autoColor = statusLabel.ForeColor;
                    this.Tag = autoColor;
                }
                statusLabel.ForeColor = autoColor;

                // Определяем порт сервера.
                int port;
                if (!int.TryParse(Environment.GetEnvironmentVariable("PIPELINE_PORT"), out port))
                    port = 5501;

                // Устанавливаем соединение с сервером.
                _tcp = new TcpClient();
                statusLabel.Text = "Connecting...";
                _tcp.Connect(IPAddress.Parse(servers.Text), port);
                if (!_tcp.Connected)
                    throw new Exception("Server unavailable");

                _Connect();
                statusLabel.Text = "Connected.";
            }
            catch (Exception ex)
            {
                // Информируем об ошибке.
                Debug.Print("{0},\n{1}", ex.Message, ex.StackTrace);
                statusLabel.Text = "Could not connect!";
                statusLabel.ForeColor = Color.Red;
                me.Enabled = true;
            }
        }

        private void Admin_FormClosed(object sender, FormClosedEventArgs e)
        {
            _tcp.Close();
        }

    }
}

