﻿namespace pipeline
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.Linq;
    using System.Text;
    using System.Threading.Tasks;

    using AustinHarris.JsonRpc;

    internal sealed class ApiAdapter : JsonRpcService
    {
        Data _data;
        PluginEngine _engine;

        public ApiAdapter(Data data)
        {
            _data = data;
        }

        [JsonRpcMethod]
        public string GetVersion()
        {
            return Plugin.Version.ToString();
        }

        [JsonRpcMethod]
        public void LoadData(string xml)
        {
            _data.Load(xml);
        }

        [JsonRpcMethod]
        public void SaveData()
        {
            _data.Save();
        }

        [JsonRpcMethod]
        public string[] GetElements()
        {
            return _data.ContentIds;
        }

        [JsonRpcMethod]
        public string GetStyle(string element)
        {
            //return _data.Find(element).Name.LocalName;
            return _data.GetStyle(_data.Find(element)).Id;
        }

        [JsonRpcMethod]
        public void SetStyle(string element, string style)
        {
            _data.SetStyle(_data.Find(element), style);
        }

        [JsonRpcMethod]
        public void Navigate(double latitude, double longitude, int scale)
        {
            var center = Data.Point.Create(_data.Engine, latitude, longitude);
            _data.Engine.Map.ShowPos(center.Local, scale, false);
        }
    }
}
