﻿namespace pipeline
{
    using GrymCore;
    using System;
    using System.Diagnostics;
    using System.Xml.Linq;

    internal abstract class Control
        : IControlAppearance
        , IControlPlacement
        , IDisposable
    {
        private string _caption = string.Empty;
        private string _description = string.Empty;
        private object _icon = null;
        private string _placementCode = string.Empty;
        private string _tag = string.Empty;

        protected Control(XElement elm)
        {
            // инициализация стандартных полей из хмл
            //имя хмл элемента может совпадать с именем конечного класса
            // если ни одного хмл элемента не найдено пропускаем иницализацию
            Debug.Assert(elm != null);

            // строим поле тага как полный путь хмл элемента
            _tag = elm.Name.LocalName;
            for (var node = elm.Parent; node != null; node = node.Parent)
                _tag = node.Name.LocalName + "." + _tag;

            Debug.Assert(elm.Attribute("caption") != null);
            _caption = elm.Attribute("caption").Value.Trim();

            Debug.Assert(elm.Attribute("description") != null);
            _description = elm.Attribute("description").Value.Trim();

            // код расположения либо простой либо расширенный (по документации GrymAPI)
            if (elm.Element("placement") == null)
            {
                Debug.Assert(elm.Attribute("placement") != null);
                _placementCode = elm.Attribute("placement").Value;
            }
            else
            {
                elm = elm.Element("control_pos");
                if (elm != null)
                    _placementCode = elm.Value;
            }
        }

        public string Caption
        {
            get { return _caption; }
            set { _caption = value; }
        }

        public string Description
        {
            get { return _description; }
            set { _description = value; }
        }

        public object Icon
        {
            get { return _icon; }
            set { _icon = value; }
        }

        public string PlacementCode
        {
            get { return _placementCode; }
            set { _placementCode = value; }
        }

        public string Tag
        {
            get { return _tag; }
            set { _tag = value; }
        }

        public virtual void Dispose()
        {
            _icon = null;
        }
    }
}

