﻿namespace pipeline
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Diagnostics;
    using System.Linq;
    using System.Text;
    using System.IO;
    using System.Threading;
    using System.Net;
    using System.Net.Sockets;
    
    using AustinHarris.JsonRpc;

    // AustinHarris JsonRpc http://jsonrpc2.codeplex.com/
    internal class JsonApi 
    {
        private bool _started = false;
        private TcpListener _server;
        private Layer _layer;
        private ApiAdapter _adapter;

        public JsonApi()
        {
        }

        public void StartAsync(PluginEngine engine, int port)
        {
            BackgroundWorker bw = new BackgroundWorker();
            bw.DoWork += new DoWorkEventHandler(delegate(object owner, DoWorkEventArgs args) {
                try
                {
                    _server = new TcpListener(IPAddress.Loopback, port);
                    _server.Start();

                    _layer = new Layer(engine);
                    _adapter = new ApiAdapter(_layer.Data);
                    _started = true;
                }
                catch (Exception ex)
                {
                    Debug.Print("{0}\n{1}", ex.Message, ex.StackTrace);
                }

                var rpc_handler = new AsyncCallback(state =>
                {
                    var async = state as JsonRpcStateAsync;
                    var result = async.Result;
                    var writer = async.AsyncState as StreamWriter;

                    Debug.Print("Result " + result);
                    writer.WriteLine(result);
                    writer.FlushAsync();
                });

                Action<StreamWriter, string> rpc_call = (writer, line) =>
                {
                    var async = new JsonRpcStateAsync(rpc_handler, writer) { JsonRpc = line };
                    Debug.Print("Line " + line);
                    JsonRpcProcessor.Process(async, writer);
                };

                while (_started) try
                {
                    Debug.Print("Waiting connections");
                    using (var client = _server.AcceptTcpClient())
                    using (var stream = client.GetStream())
                    {
                        Debug.Print("Client Connected");
                        var stream_reader = new StreamReader(stream, Encoding.UTF8);
                        var stream_writer = new StreamWriter(stream, new UTF8Encoding(false));
                        
                        Debug.Print("Result");
                        while (!stream_reader.EndOfStream)
                        {
                            Debug.Print("Reading");
                            var line = stream_reader.ReadLine().Trim();
                            rpc_call(stream_writer, line);
                            Debug.Print("Complete");
                        }
                        Debug.Print("Finish");
                    }
                }
                catch (Exception ex)
                {
                    Debug.Print("{0}\n{1}", ex.Message, ex.StackTrace);
                    Stop();
                    break;
                }
            });

            bw.RunWorkerAsync();
        }

        public void Stop()
        {
            if (_server == null)
                return;

            Debug.Print("Closing");

            if (_started)
            {
                _started = false;
                _server.Stop();
            }

            if (_layer != null)
            {
                _layer.Dispose();
                _layer = null;
            }

            GC.SuppressFinalize(this);
            GC.Collect();
            GC.WaitForPendingFinalizers();
        }
    }
}
