﻿namespace pipeline
{
    using GrymCore;
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.Globalization;
    using System.IO;
    using System.Runtime.InteropServices;
    using System.Text;
    using System.Threading;
    using System.Windows.Forms;
    using System.Xml.Linq;

    using Microsoft.Win32;
    using AustinHarris.JsonRpc;

    internal sealed class Layer :
        Control,
        IPluginCustomDrawLayer,
        ILayer,
        IChildControl
    {
        private Data _data;
        private Render _render;
        private bool _isVisible = true;
        private Painter _painter = new Painter();

        public Layer(PluginEngine engine) : base(XDocument.Parse(Properties.Resources.layerXml).Root)
        {
            _data = new KmlData(engine);
            _render = new Render(this);
            engine.Map.Layers.AddLayer(this);
        }

        public Data Data
        {
            get
            {
                return _data;
            }
        }

        public void Update(string id, string style)
        {
            if (id.Length == 0 || style.Length == 0)
                throw new Exception(string.Format("Invalid argument: id = {0}, style = {1}", id, style));

            Debug.Print("Updating id = {0} with style = {1}", id, style);
            
            var node = Data.Find(id);
            if (node == null)
                throw new Exception("Unknown element " + id);
            
            Data.SetStyle(node, style);
            Data.Engine.Map.Invalidate(true);
        }

        #region JsonRpcService Members
        
        #endregion

        #region IDisposable Members

        public override void Dispose()
        {
            base.Dispose();
            Data.Engine.Map.Layers.RemoveLayer(this);
        }

        #endregion

        #region IPluginCustomDrawLayer Members

        public void Draw(IMapDevice device)
        {
            if (DeviceType.DeviceTypeWindow != device.Type)
                return;
            
            IMapRect drawRect = Data.Engine.Map.GetMapVisibleRect(); // Видимая часть карты
            IMapRect fullRect = Data.Engine.Map.FullExtent; // Вся карта
            _painter.Scale = (drawRect.Width * drawRect.Height) / (fullRect.Width * fullRect.Height); // Отношение площадей
            _painter.Hdc = (IntPtr)device.SafeDC;

            //Debug.Print("Corners ({0},{1}) ({2},{3}) ({4},{5}) ({6},{7})", corners[0].X, corners[0].Y, corners[1].X, corners[1].Y, corners[2].X, corners[2].Y, corners[3].X, corners[3].Y);

            _render.Draw(Data.Find(Data.ContentIds), _painter, device, drawRect);
        }

        #endregion

        #region ILayer Members

        public bool VisibleState
        {
            get { return _isVisible; }
            set { _isVisible = value; }
        }

        public bool CheckVisible(int scale, DeviceType deviceType)
        {
            //Debug.Print("Scale = {0}, DeviceType = {1}", scale, (int)deviceType);
            return VisibleState && scale >= 0;
        }

        #endregion


        #region IChildControl Members

        public object ParentControl { get { return null; } }
        
        #endregion
    }
}



