﻿namespace pipeline
{
    using GrymCore;
    using System;
    using System.Diagnostics;
    using System.Drawing;
    using System.Runtime.InteropServices;
    using Properties;

    internal class Painter
    {
        private static readonly Font _DEFAULT_FONT = new Font(FontFamily.GenericMonospace, 8);
        private static readonly SolidBrush _DEFAULT_BRUSH = new SolidBrush(Color.Black);

        private IntPtr _hdc;
        private double _scale = 1.0;
        private Brush _brush = _DEFAULT_BRUSH;
        private Pen _pen = new Pen(_DEFAULT_BRUSH);

        private static float _Double2Float(double value)
        {
            float rv = (float) value;
            if (float.IsPositiveInfinity(rv))
            {
                rv = float.MaxValue;
            }
            else if (float.IsNegativeInfinity(rv))
            {
                rv = float.MinValue;
            }

            return rv;
        }
        public static Point ImageOffset(Image image)
        {
            Point rv = Point.Empty;
            rv.X = -image.Size.Width / 2;
            rv.Y = -image.Size.Height / 2;

            return rv;
        }

        public double Scale
        {
            get { return _scale;  }
            set { _scale = value;  }
        }

        public Pen Pen
        {
            get { return _pen; }
        }

        public IntPtr Hdc
        {
            set { _hdc = value; }
        }

        public void Draw(IDevPoint point, string text, Font font = null)
        {
            Debug.Assert((_hdc != null) && (point != null) && (text.Length != 0));

            if (font == null)
                font = _DEFAULT_FONT;

            using (Graphics graphics = Graphics.FromHdc(_hdc))
            {
                graphics.DrawString(text, font, _brush, new Point(point.X, point.Y));
            }
        }

        public void Draw(IDevPoint point, Image image)
        {
            Debug.Assert((_hdc != null) && (point != null) && (image != null));

            using (Graphics graphics = Graphics.FromHdc(_hdc))
            {
                var unit = GraphicsUnit.Pixel;
                var rect = image.GetBounds(ref unit);

                /*
                 * Scale == 1.0 => target = {     2.0,         2.0      }
                 * Scale == 0.0 => target = { image.Width, image.Height }
                 */


                //var bak = RectangleF.Inflate(rect, 0.0f, 0.0f);
                if (Scale < 1.0)
                {
                    rect.Width -= rect.Width * _Double2Float(Scale);
                    rect.Height -= rect.Height * _Double2Float(Scale);
                }
                else
                {
                    rect.Width = 2.0f;
                    rect.Height = 2.0f;
                }

                rect.X = point.X - rect.Width / 2.0f;
                rect.Y = point.Y - rect.Height / 2.0f;
                
                graphics.DrawImage(image, rect);

                //Debug.Print("{0} - ({1}) ({2})", Scale, bak, rect);
            }
        }

        public void Draw(IDevPoint[] points)
        {
            Debug.Assert((_hdc != null) && (points != null) && (points.Length > 1));

            var local = new Point[points.Length];
            for (var i = 0; i < local.Length; ++i)
            {
                local[i].X = points[i].X;// Math.Max(points[i].X, 0);
                local[i].Y = points[i].Y;// Math.Max(points[i].Y, 0);
            }

            using (Graphics graphics = Graphics.FromHdc(_hdc))
            {
                graphics.DrawLines(_pen, local);
            }
        }
    }
}

