﻿namespace pipeline
{
    using GrymCore;
    using System;
    using System.Drawing;
    using System.Diagnostics;
    using System.Drawing.Imaging;
    using System.IO;
    using System.Net;
    using System.Collections;
    using System.Collections.Generic;
    using System.Xml.Linq;
    using System.Linq;
    using System.Windows.Forms;
    using System.Runtime.InteropServices;
    
    internal sealed class PluginEngine : IDisposable
    {
        private string _cityName;
        private string _kmlPath;
        private IBaseViewThread _baseView;

        public PluginEngine(IBaseViewThread baseView)
        {
            _baseView = baseView;

            FileInfo fi = new FileInfo(baseView.BaseReference.FullPath);
            _cityName = fi.Name.Replace("Data_", string.Empty).Replace(fi.Extension, string.Empty);

            Trace.WriteLine("PluginEngine created");
        }

        public IMap2 Map
        {
            get { return (_baseView.Frame.Map as IMap2); }
        }

        
        public IntPtr Handle 
        { 
            get { return new IntPtr(_baseView.Frame.HWindow); }
        }

        public string KmlPath
        {
            get { return _kmlPath; }
            set { _kmlPath = value; }
        }

        public IRaster LoadRaster(string resourceName)
        {
            var bmp = Properties.Resources.ResourceManager.GetObject(resourceName, Properties.Resources.Culture) as System.Drawing.Bitmap;
            return LoadRaster(bmp);
        }

        public IRaster LoadRaster(Bitmap bmp)
        {
            if (bmp == null)
                return null;

            using (MemoryStream mem = new MemoryStream())
            {
                bmp.Save(mem, ImageFormat.Png);
                return Factory.CreateRasterFromMemory(mem.GetBuffer());
            }
        }

        public string CityName
        {
            get { return _cityName; }
        }

        public IBaseViewThread BaseView
        {
            get { return _baseView; }
        }

        public IMapCoordinateTransformationGeo CoordinateTransformation
        {
            get { return (Map.CoordinateTransformation as IMapCoordinateTransformationGeo); }
        }

        public IGrymObjectFactory3 Factory
        {
            get { return (_baseView.Factory as IGrymObjectFactory3); }
        }


        public void Dispose()
        {
            if (_baseView != null)
                Marshal.FinalReleaseComObject(_baseView);

            _baseView = null;
        }

    }
}

